@@ -8,6 +8,7 @@ use crate::proto::gen::transforms::TransformPipeline;
 use crate::spec::chart::{ChartSpec, ChartVisitor};
 use crate::spec::data::{DataFormatParseSpec, DataSpec};
 use crate::spec::mark::{MarkFacetSpec, MarkSpec};
+use crate::spec::projection::ProjectionSpec;
 use crate::spec::scale::{
     ScaleArrayElementSpec, ScaleBinsSpec, ScaleDataReferenceOrSignalSpec, ScaleDomainSpec,
     ScaleFieldReferenceSpec, ScaleRangeSpec, ScaleSpec,
@@ -63,6 +64,12 @@ impl ChartVisitor for MakeTaskScopeVisitor {
         Ok(())
     }
 
+    fn visit_projection(&mut self, projection: &ProjectionSpec, scope: &[u32]) -> Result<()> {
+        let task_scope = self.task_scope.get_child_mut(scope)?;
+        task_scope.scales.insert(projection.name.clone());
+        Ok(())
+    }
+
     fn visit_group_mark(&mut self, mark: &MarkSpec, scope: &[u32]) -> Result<()> {
         // Initialize scope for this group level
         let parent_scope = self.task_scope.get_child_mut(&scope[0..scope.len() - 1])?;
@@ -252,6 +259,10 @@ impl<'a> ChartVisitor for MakeTasksVisitor<'a> {
     fn visit_scale(&mut self, _scale: &ScaleSpec, _scope: &[u32]) -> Result<()> {
         unimplemented!("Scale tasks not yet supported")
     }
+
+    fn visit_projection(&mut self, _projection: &ProjectionSpec, _scope: &[u32]) -> Result<()> {
+        unimplemented!("Projection tasks not yet supported")
+    }
 }
 
 #[derive(Clone, Debug, Default)]
@@ -285,6 +296,12 @@ impl ChartVisitor for DefinitionVarsChartVisitor {
             .insert((Variable::new_scale(&scale.name), Vec::from(scope)));
         Ok(())
     }
+
+    fn visit_projection(&mut self, projection: &ProjectionSpec, scope: &[u32]) -> Result<()> {
+        self.definition_vars
+            .insert((Variable::new_scale(&projection.name), Vec::from(scope)));
+        Ok(())
+    }
 }
 
 /// Collect "update variables" from the spec. These are variables that are updated somewhere other
@@ -334,9 +351,9 @@ impl<'a> ChartVisitor for UpdateVarsChartVisitor<'a> {
 
     fn visit_signal(&mut self, signal: &SignalSpec, scope: &[u32]) -> Result<()> {
         // Signal is an update variable if it's not an empty stub
-        if signal.value.as_option().is_some()
-            || signal.init.is_some()
+        if signal.init.is_some()
             || signal.update.is_some()
+            || signal.bind.is_some()
             || !signal.on.is_empty()
         {
             self.update_vars
@@ -395,6 +412,13 @@ impl<'a> ChartVisitor for UpdateVarsChartVisitor<'a> {
             .insert((Variable::new_scale(&scale.name), Vec::from(scope)));
         Ok(())
     }
+
+    fn visit_projection(&mut self, projection: &ProjectionSpec, scope: &[u32]) -> Result<()> {
+        // See scales note
+        self.update_vars
+            .insert((Variable::new_scale(&projection.name), Vec::from(scope)));
+        Ok(())
+    }
 }
 
 #[derive(Clone, Debug)]
